import * as Messages from "/services/messages/messages.js";

let electionModel;

export function getElectionModel() {
  if (electionModel == null) {
    electionModel = new ElectionModel();
  }
  return electionModel;
}

class ElectionModel {
  constructor() {}

  async getElection(id) {
    if (this.elections == null) await this.refreshElections();
    let electionToGet;
    this.elections.forEach((election) => {
      if (election.ID == id) electionToGet = election;
    });
    return electionToGet;
  }

  async getElections() {
    if (this.elections == null) {
      try {
        const response = await fetch("/api/Election/", {
          method: "GET",
          headers: new Headers({
            "XSRF-Token": this.current_user.xsrftoken,
          }),
        });
        if (response.status !== 200) {
          throw new Error(
            `Elections could not be fetched (status ${response.status})`
          );
        }
        this.elections = await response.json();
      } catch (e) {
        Messages.Show("is-warning", e.message);
        console.error(e);
      }
    }
    return this.elections;
  }

  async saveElection(
    method,
    ID,
    Name,
    BallotType,
    MapAreaFile,
    MapSectionFile
  ) {
    try {
      const response = await fetch("/api/Election/" + ID, {
        method: method,
        headers: new Headers({
          "XSRF-Token": this.current_user.xsrftoken,
        }),
        body: JSON.stringify({
          ID: ID,
          Name: Name,
          BallotType: BallotType,
          MapAreaFile: MapAreaFile,
          MapSectionFile: MapSectionFile,
        }),
      });
      if (response.status == 409) {
        throw new Error(
          `The name of the election already exist (status ${response.status})`
        );
      }
      if (response.status !== 200) {
        throw new Error(
          `Election could not be updated or created (status ${response.status})`
        );
      }
      this.refreshElections();
      return await response.json();
    } catch (e) {
      Messages.Show("is-warning", e.message);
      console.error(e);
      return;
    }
  }

  async deleteElection(ID) {
    try {
      const response = await fetch("/api/Election/" + ID, {
        method: "delete",
        headers: new Headers({
          "XSRF-Token": this.current_user.xsrftoken,
        }),
      });
      if (response.status !== 200) {
        throw new Error(
          `Election could not be deleted (status ${response.status})`
        );
      }
      this.refreshElections();
    } catch (e) {
      Messages.Show("is-warning", e.message);
      console.error(e);
    }
  }

  async refreshElections() {
    this.elections = null;
    await this.getElections();
  }

  checkInputs(method, ID, Name, BallotType) {
    if (method == "PUT" && ID == "") {
      Messages.Show(
        "is-warning",
        "Un erreur s'est produite lors de la mise à jour."
      );
      return false;
    }
    if (Name == "") {
      Messages.Show("is-warning", "Veuillez choisir un nom pour l'élection");
      return false;
    }
    if (BallotType == "") {
      Messages.Show(
        "is-warning",
        "Veuillez choisir un type de scrutin pour l'élection"
      );
      return false;
    }
    return true;
  }
}
